;;; igrep-find-replace.el --- igrep interface for query-replace

;; $Id: igrep-find-query-replace.el,v 1.9 2003/01/10 05:57:59 burton Exp $

;; Copyright (C) 2000-2003 Free Software Foundation, Inc.
;; Copyright (C) 2000-2003 Kevin A. Burton (burton@openprivacy.org)

;; Author: Kevin A. Burton (burton@openprivacy.org)
;; Maintainer: Kevin A. Burton (burton@openprivacy.org)
;; Location: http://relativity.yi.org
;; Keywords:
;; Version: 1.0.3

;; This file is [not yet] part of GNU Emacs.

;; This program is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free Software
;; Foundation; either version 2 of the License, or any later version.
;;
;; This program is distributed in the hope that it will be useful, but WITHOUT
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License along with
;; this program; if not, write to the Free Software Foundation, Inc., 59 Temple
;; Place - Suite 330, Boston, MA 02111-1307, USA.

;;; Commentary:
;;
;; This is a utility for igrep.  It allows you to run igrep-find and
;; 'query-replace' within all matched files.  This allows you to make major
;; changes to multiple files at once.
;;
;; The interface behaves just like 'igrep' and 'query-replace' and shouldn't
;; confuse the user with any new UI metaphor.
;;

;;; History
;;
;; - Sat Dec 15 2001 10:58 PM (burton@openprivacy.org): we now log all sentinel
;; events in the *igrep* buffer.
;; 
;; - Sat Dec 15 2001 10:42 PM (burton@openprivacy.org): FIXED BUG: when the
;; match string is LONGER than the search string we need to update the region..
;;
;; - Wed Nov 07 2001 12:08 AM (burton@openprivacy.org): need to reset the
;; overlay and overlay-arrow when igrep-find is done.

;; Version 1.0.3
;;
;; - Wed Oct 31 2001 10:38 PM (burton@openprivacy.org): overlay-arrow-string
;; must be non-nil.  AKA '>'
;;
;; - Wed Oct 31 2001 09:24 AM (burton@openprivacy.org): need to show the user
;;   (somehow) that the replacement was actually done correctly.  This is
;;   necessary because we need to show the user what is actually happening
;;   without any confusion.
;;
;;     - now using sit-for to handle this.
;;
;; - Tue Oct 30 2001 07:26 PM (burton@openprivacy.org): we now display the
;; process termination event.
;;
;; - Sat Oct 27 2001 08:32 PM (burton@openprivacy.org): we are now using an
;; 'overlay-arrow' in the igrep buffer.
;;
;; - Sat Oct 27 2001 07:15 PM (burton@openprivacy.org): message stating that
;; igrep-find-query-will start when igrep finishes.
;;
;; - Thu Oct 25 2001 01:15 AM (burton@openprivacy.org): need a "y or n" question
;;   on the query prompt.
;;

;;; TODO:
;;
;;
;;; HIGH PRIORITY...

;; - I can NOT C-g out of igrep-find-query-replace... why is this???

;; - when the file buffer (not the *igrep* buffer) shows the match near the end
;;   of beginning of the buffer, center the window so that it is more obvious what
;;   we are replacing.
;;
;;     - Use (recenter) to do this!  Come up with a use case that fails here and
;;       fix it.
;;
;; - Make sure that the it is obvious what buffer we are in in the *igrep*
;;   buffer.
;;
;;     - use the secondary selection face to highlight the current line (in the
;;       *igrep* buffer) I am replacing.
;; 
;; - support the ability to replace multi-line text..
;;
;; - is it possible to stop using the *igrep* buffer and use an
;; *igrep-find-query-replace* buffer instead?

;;; LOW PRIORITY...

;; 

;; - implement igrep-query-replace as well as igrep-find-query-replace.  Any
;;   other functions I should implement??  agrep, fgrep, etc?
;;
;; - for some reason I am getting this error:
;;
;;     error in process sentinel: Args out of range: 0, 6 [2 times]
;;
;; - Use an italic overlay to highlight the current line/file.
;;
;; - In long buffers, keep the current line near the top of the buffer.

(require 'igrep)

;;; Code:
(defvar ifqr-from nil "String to search and replace from.")
(make-variable-buffer-local 'ifqr-from)

(defvar ifqr-with nil "String to search and replace with.")
(make-variable-buffer-local 'ifqr-with)

(defvar ifqr-replacement-count 0 "Number of replacements for the last `query-replace'.")

(defvar ifqr-text-buffer-name "*ifqr-text*" "Buffer used for reading and writing
text/multiline string")

(defvar ifqr-text-replace-from nil "Search for the given text.")

(defvar ifqr-text-replace-with nil "Replace the given text.")

(defvar ifqr-text-replace-var-name nil "Var name to set.")

(defvar ifqr-text-replace-function nil "Function to eval after completion.")

(defface ifqr-line-overlay-face nil "Overlay used to highlight the current match.")
(set-face-background 'ifqr-line-overlay-face (face-background 'secondary-selection))

(defvar ifqr-line-overlay (make-overlay 0 0) "Overlay used to highlight this match.")

(defun igrep-find-query-replace(replace-from replace-with files)
  "Use igrep-find to query and replace strings within buffers."
  (interactive

   (let(from to files)
     (setq from (read-string "Query replace: "))

     (setq to (read-string (format "Query replace %s with: " from )))

     ;;use igrep-read-files so that it is similar to igrep
     (setq files (igrep-read-files))

     (list from to files)))

  (delete-other-windows)
  
  (let ((igrep-find t)
        igrep-args
        filename
        process)
    
    (setq igrep-args (list "grep" replace-from files))

    ;;run igrep-find...
    (igrep (elt igrep-args 0)
           (elt igrep-args 1)
           (elt igrep-args 2))
    
    (message "igrep-find-query-replace will start when igrep finishes...")

    ;;get the process created by igrep
    (setq process (get-process "igrep"))

    ;;if the process is nil... it exited quickly... process the output.
    (if process
        ;; use a process sentinel to find it.
        (progn

          ;;update buffer local variables for from/width
          (save-excursion

            (set-buffer (process-buffer process))

            (setq ifqr-from replace-from)

            (setq ifqr-with replace-with))
          
          (set-process-sentinel process 'ifqr-sentinel))

      ;;this should never happen... I think... maybe we shouldn't signal an
      ;;error though.
      (error "Unable to find process"))))

(defun ifqr-do()
  "Do the query/replace after igrep has ran saving.  Show the number of
replacements when complete. "

  ;;reset the count
  (setq ifqr-replacement-count 0)
    
  (let(filename line-number replace-from replace-with quit)
      
    (set-buffer "*igrep*")
      
    (display-buffer "*igrep*")
      
    ;;get the local replacement vars..
    (setq replace-from ifqr-from)

    (setq replace-with ifqr-with)
      
    (goto-char (point-min))

    (forward-line 2)

    (ifqr-highlight-line (point))
    
    ;;navigate forward though the igrep buffer.
    (while (and (not quit)
                (re-search-forward "\\(^/[^:]+\\):\\([0-9]+\\):" nil t))

      (ifqr-highlight-line (point))
        
      (setq filename (match-string-no-properties 1))

      (setq line-number (string-to-number (match-string-no-properties 2)))
      
      (setq quit (ifqr-do-replace filename line-number replace-from replace-with))

      ;;return to the igrep buffer just in case.
      (set-buffer "*igrep*")

      (goto-char (point-at-eol))
      (forward-line 1)
      (goto-char (point-at-bol)))

    (ifqr-highlight-reset)
    
    (message "Replaced %i occurrences...done" ifqr-replacement-count)))

(defun ifqr-highlight-reset()
  "Reset any highlights in the *igrep* buffer."

  (save-excursion

    (set-buffer "*igrep*")

    (setq overlay-arrow-position nil)))

(defun ifqr-do-replace(filename line-number replace-from replace-with)
  "Do a replacement on the given line number, keep doing until we are complete.
If the user select 'q' (for quit) return true, else nil. "

  ;;valid options are y/n/q
  (let(source-buffer result)

    (setq source-buffer (find-file-noselect filename))

    (set-window-buffer (other-window 0) source-buffer)
    
    (set-buffer source-buffer)

    ;;goto-line has problems with strings.
    (if (stringp line-number)
        (setq line-number (string-to-number line-number)))
    
    (goto-line line-number)

    (goto-char (point-at-bol))

    (if (search-forward replace-from nil t)
        (condition-case nil
            (progn

             ;;if an error is signaled... catch it... this can happen from mouse
              ;;events/etc..

              ;;highlight the match...
              (ifqr-highlight (match-beginning 0) (match-end 0))

              ;;should we replace
              (if (yes-or-no-p (format "Query replacing '%s' with '%s' in %s: " replace-from replace-with filename))
                  (progn

                    (replace-match replace-with)

                    (ifqr-highlight-entire-match replace-with)
                
                    (sit-for .2)
                
                    ;;increment the match count...
                    (setq ifqr-replacement-count (1+ ifqr-replacement-count))
                
                    ;;important to save the buffer when we are done.
                    (save-buffer))))
          nil)
      (message (format "Could not find %s in %s on line number %i within buffer %s"
                       replace-from
                       filename
                       line-number
                       (buffer-name))))
    result))

(defun ifqr-highlight(region-start region-end)
  "Setup the mark within the given region."

  (goto-char region-start)
  (push-mark nil t t)
  (goto-char region-end))

(defun ifqr-highlight-entire-match(replace-with)
  "Highlight the entire match."

  (goto-char (match-beginning 0))

  (search-forward replace-with)
  
  (ifqr-highlight (match-beginning 0) (match-end 0)))

(defun ifqr-sentinel(process event)
  "Process sentinel which detects if the igrep process is done."

  ;;ok.  process finished. we can keep going
  (when(string-match "^finished\n$" event)
    (ifqr-do))

  ;;ok.  if we exit abnormally... in some situations it is OK to keep going.
  (when (string-match "^exited abnormally" event)

    (save-excursion

      (set-buffer "*igrep*")

      ;;we can keep going if there is greather than two lines
      (if (> (count-lines (point-min) (point-max)) 2)
          (ifqr-do))))

  ;;always output the event
  (save-excursion
    
    (set-buffer "*igrep*")
    
    (goto-char (point-max))
    
    (insert event)))

(defun igrep-find-query-replace-text()
  "Run query replace on long/text strings."
  (interactive)

  (ifqr-text-step-1))

(defun ifqr-text-step-1()
  "Get the from variable."
  
  (ifqr-read-text-string 'ifqr-text-replace-from
                         'ifqr-text-step-2
                         "(Step one) Enter the text you would like to search for."))

(defun ifqr-text-step-2()

  (ifqr-read-text-string 'ifqr-text-replace-with
                         'ifqr-text-do
                         "(Step two) Enter the text you would like to use as a replacement."))

(defun ifqr-read-text-string(var-name function message)
  "Read a text string from a buffer."

  (setq ifqr-text-replace-var-name var-name)

  (setq ifqr-text-replace-function function)
  
  (save-excursion

    (set-buffer (get-buffer-create ifqr-text-buffer-name))

    (erase-buffer)
    
    (insert (format "//REPLACE-TEXT: %s\n" message))

    (ifqr-text-entry-mode))

  (pop-to-buffer ifqr-text-buffer-name)
    
  (message message))

(defun ifqr-read-text-complete()
  "Complete text entry and continue."
  (interactive)

  (ifqr-text-cleanse-buffer)

  (delete-window (get-buffer-window ifqr-text-buffer-name))

  (funcall ifqr-text-replace-function))

(defun ifqr-text-cleanse-buffer()
  
  (save-excursion
    (set-buffer ifqr-text-buffer-name)

    (goto-char (point-min))

    (while (re-search-forward "^//REPLACE-TEXT: .*$" nil t)
      (delete-region (match-beginning 0)
                     (1+ (match-end 0))))))

(defun ifqr-highlight-line(point)
  "Highlight the current line so that people know what the heck is going on..."
  (interactive
   (list
    (point)))
  
  (save-excursion

    (set-buffer "*igrep*")

    (let((redisplay-dont-pause t)
         (begin nil)
         (end nil))

      (goto-char point)                 

      (setq begin (point-at-bol))

      (setq end (1+ (point-at-eol)))

      ;;update the Emacs 21 arrow...
      (let(m)
        (setq m (make-marker))

        (set-marker m begin)

        (make-local-variable 'overlay-arrow-string)
        (setq overlay-arrow-string "=>")
        
        (setq overlay-arrow-position m))
      
      (move-overlay ifqr-line-overlay begin end (current-buffer))
      
      (overlay-put ifqr-line-overlay 'face 'ifqr-line-overlay-face)
      
      (overlay-put ifqr-line-overlay 'window (selected-window))

      (overlay-put ifqr-line-overlay 'priority 1)

      (sit-for 0))))

(define-derived-mode ifqr-text-entry-mode fundamental-mode "IGrepTextEntry"
  "Major mode with key bindings to jump to files."
  (interactive)

  (font-lock-mode 1))

;;comments
(font-lock-add-keywords 'ifqr-text-entry-mode '(("^//.*$"
                                                 (0 'font-lock-comment-face t))))

(define-key ifqr-text-entry-mode-map "\C-c\C-c" 'ifqr-read-text-complete)

(provide 'igrep-find-query-replace)

;;; igrep-find-replace.el ends here
