;;; space-indent.el --- Forced space indentions for GNU Emacs

;; $Id: space-indent.el,v 1.3 2003/01/10 05:57:59 burton Exp $

;; Copyright (C) 2000-2003 Free Software Foundation, Inc.
;; Copyright (C) 2000-2003 Kevin A. Burton (burton@openprivacy.org)

;; Author: Kevin A. Burton (burton@openprivacy.org)
;; Maintainer: Kevin A. Burton (burton@openprivacy.org)
;; Location: http://relativity.yi.org
;; Keywords:
;; Version: 1.0.1

;; This file is [not yet] part of GNU Emacs.

;; This program is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free Software
;; Foundation; either version 2 of the License, or any later version.
;;
;; This program is distributed in the hope that it will be useful, but WITHOUT
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License along with
;; this program; if not, write to the Free Software Foundation, Inc., 59 Temple
;; Place - Suite 330, Boston, MA 02111-1307, USA.

;;; Commentary:
;;
;; I need a way to do forced indentions with space characters independent of
;; major modes.  I have had this feature in other editors but never in Emacs.
;; Basically this just allows you to move/justify huge regions of text with one
;; key binding.
;;
;; This provides a slight duplication of effort with the core Emacs but I
;; believe this is somewhat modular and straight forward.
;;
;;; Install:
;;
;; Just add
;;
;; (require 'space-indent)
;;
;; in your.emacs file.

;;; TODO:
;;

;;; Code:

;;; History: 

;; Tue Sep 04 2001 04:57 PM (burton@relativity.yi.org): Now supporting a smarter
;; regexp so that text can be aligned all the way flush with the left.

(defcustom space-indent-width 4
  "Number of spaces to indent regions."
  :type 'integer
  :group 'space-indent)
    
(defun space-indent-right(start end)
  "Indent the given region one level to the right."
  (interactive "r")

  (space-indent--facade 'right start end))
    
(defun space-indent-left(start end)
  "Indent the given region one level to the right."
  (interactive "r")

  (space-indent--facade 'left start end))

(defun space-indent--facade(direction start end)
  "Facade so that `space-indent-right' and `space-ident-left' can share
behavior."

  ;;only use the current line if the mark is not active...
  (if (null mark-active)
      (save-excursion

        (goto-char (point-at-bol))

        (setq start (point))

        (goto-char (point-at-eol))
        
        (setq end (point))))
        
      (space-indent--do direction start end))

(defun space-indent--do(direction start end)
  "Do indentation..."

  (save-restriction

    (narrow-to-region start end)

    (if (equal direction 'right)
        (space-indent--do-right)
      (space-indent--do-left))
    
    ;;now update the region

    (setq start (point-min))
    
    (setq end (point-max)))

  (setq deactivate-mark nil)
  (setq mark-active t)

  ;;FIXME: is it possible to set the mark without changing the point???

  (let((orig-point (point)))
    
    (goto-char start)
    
    (push-mark end t t)))

(defun space-indent--do-right()

  (goto-char (point-min))

  (while (re-search-forward "^." nil t)

    ;;if we are not looking at a null line..

    (if (not (looking-at "^$"))
        (progn 
          (goto-char (match-beginning 0))
          
          (insert (space-indent--string))))))

(defun space-indent--do-left()

  (goto-char (point-min))

  (while (re-search-forward (space-indent--regexp) nil t)
    (delete-region (match-beginning 0) (match-end 0))

    (goto-char (point-at-eol))))

(defun space-indent--string()
  "String to use for indentation.  A set of spaces."

  (make-string space-indent-width ? ))

(defun space-indent--regexp()
  "Build a regexp for searching for spaces."

  (let((i 1)
       (regexp "^[ ]"))

    (while (< i space-indent-width)

      (setq regexp (concat regexp "[ ]?"))

      (setq i (1+ i)))

    regexp))
  
;;  (concat "^" (space-indent--string)))
  
;;define keys to work with this.

(define-key global-map [\C-\M-right] 'space-indent-right)
(define-key global-map [\C-\M-left] 'space-indent-left)

(provide 'space-indent)

;;; space-indent.el ends here
