;;; thesaurus.el --- Emacs integration for the 1911 Roget's Thesaurus.

;; $Id: thesaurus.el,v 1.5 2003/01/10 05:57:59 burton Exp $

;; Copyright (C) 2000-2003 Free Software Foundation, Inc.
;; Copyright (C) 2000-2003 Kevin A. Burton (burton@openprivacy.org)

;; Author: Kevin A. Burton (burton@openprivacy.org)
;; Maintainer: Kevin A. Burton (burton@openprivacy.org)
;; Location: http://relativity.yi.org
;; Keywords:
;; Version: 1.0.0

;; This file is [not yet] part of GNU Emacs.

;; This program is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free Software
;; Foundation; either version 2 of the License, or any later version.
;;
;; This program is distributed in the hope that it will be useful, but WITHOUT
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License along with
;; this program; if not, write to the Free Software Foundation, Inc., 59 Temple
;; Place - Suite 330, Boston, MA 02111-1307, USA.

;;; Commentary:

;;; TODO:
;;
;; - need code to download and uncompress the thesaurus.
;;
;; - "uniformity" returns twice :(

;;; Code:
(defvar thesaurus-data-buffer-name "*thesaurus-data*"
  "Buffer used to hold the thesaurus data.")

(defvar thesaurus-entry-buffer-name "*thesaurus-entry*"
  "Buffer used to hold a thesaurus entry.")

(defvar thesaurus-result-list-buffer-name "*thesaurus-result-list*"
  "Buffer holding the results of a query.")

;;TODO: migrate this to using the .zip format so that we save on bandwidth

(defvar thesaurus-data-url "ftp://ibiblio.org/pub/docs/books/gutenberg/etext91/roget15a.zip"
  "URL containing the Gutenberg repository for Roget's 1991 Thesaurus.")

(defvar thesaurus-data-file "~/.thesaurus-data"
  "Data file containing thesaurus definitions. Warning. This file is 1.5M
uncompressed so you will see a rise in your Emacs memory usage.")

(defvar thesaurus-current-entry nil "Current entry number in the
`thesaurus-entry-buffer-name' buffer.")

;;(make-variable-buffer-local 'thesaurus-current-entry)

(defun thesaurus-download()
  "Download the Roget's thesaurus if it isn't available."
  (interactive) ;;FIXME: 

  (if (and (not (file-exists-p thesaurus-data-file))
           (yes-or-no-p "Roget's Thesaurus is not available on disk, download? "))
      (progn

        (w3-download-url thesaurus-data-url (concat thesaurus-data-file ".zip"))

        ;;now unzip the file
        
        ))
  
  )


(defun thesaurus(word)
  "Look up a word in the thesaurus."
  (interactive
   (list
    (read-string "Word: " (current-word))))

  ;;make sure the buffer exits

  (if (not (get-buffer thesaurus-data-buffer-name))

      (progn
        (message "Creating thesaurus buffer")

        (set-buffer (get-buffer-create thesaurus-data-buffer-name))

        (insert-file thesaurus-data-file)
        
        ;;rubba dub dub... time to wash the data...
        (thesaurus-wash-data)))

  ;;need to clear the results before we go on.
  (thesaurus-result-list-clear)
  
  ;;redundant the first time because it is set in the creation, doesn't hurt
  ;;anything.
  (set-buffer thesaurus-data-buffer-name)

  (goto-char (point-min))
  
  (while (re-search-forward word nil t)

    (let(header)

      (setq header (thesaurus-get-match-header))

      (thesaurus-result-list-add header)
      
      ;;we need to move to the next entry because this word was a hit and might
      ;;show up twice in the same entry.
      (thesaurus-goto-next-entry)))

  ;;if no results were found correct the buffer
  (set-buffer (get-buffer thesaurus-result-list-buffer-name))
  (if (equal 0 (buffer-size))
      (error "No results found for: %s" word))
  
  (pop-to-buffer thesaurus-result-list-buffer-name)
  (thesaurus-result-list-mode))

(defun thesaurus-result-list-add(entry)
  "Add an entry to the current result list."

  (save-excursion
    (set-buffer (get-buffer-create thesaurus-result-list-buffer-name))

    (insert entry "\n")))

(defun thesaurus-result-list-clear()
  "Clear the current result list."
  
  (save-excursion
    (set-buffer (get-buffer-create thesaurus-result-list-buffer-name))

    (toggle-read-only -1)
    
    (erase-buffer)))
  

(defun thesaurus-get-match-header()
  "Get the first line of the matched record."

  (save-excursion

    (assert (re-search-backward "^     \\(#[0-9]+\\. \\)" nil t)
            nil "Could not find match header.")

    (buffer-substring (match-beginning 1) (point-at-eol))))

(defun thesaurus-goto-next-entry()
  "Move the point ot the beginning of the next entry in the data buffer."

  (re-search-forward "^     #[0-9]+" nil t))
  

(defun thesaurus-result-list-open-or-scroll-at-point()
  "Open the thesaurus entry at the current point."
  (interactive)

  (save-excursion

    (goto-char (point-at-bol))

    (assert (re-search-forward "^#\\([0-9]+\\)" nil t)
            nil "Could not find thesaurus entry.")

    (let(number)

      ;;if number is already current in this buffer scroll it.

      (setq number (match-string 1))

      (if (equal thesaurus-current-entry number)
          (save-excursion
            
            (display-buffer thesaurus-entry-buffer-name)
            
            (other-window -1)
            
            (scroll-up 1)

            (other-window 1))
        
        (thesaurus-view-entry number)))))

(defun thesaurus-view-entry(number)
  "Show the given entry number within thesaurus."

  (save-excursion
    (let(entry start end)

      (set-buffer thesaurus-data-buffer-name)

      (goto-char (point-min))
      
      ;;find the header.
      (assert (re-search-forward (concat "^     #" number) nil t)
              nil "Could not find the entry for number: %s" number)

      (setq start (match-beginning 0))

      ;;all records are terminated by a null line
      (assert (re-search-forward "^$" nil t)
              nil "Could not find end of entry.")

      (setq end (match-beginning 0))

      (setq entry (buffer-substring start end))

      (set-buffer (get-buffer-create thesaurus-entry-buffer-name))

      (toggle-read-only -1)
      
      (erase-buffer)
      
      (insert entry)))

  (set-buffer thesaurus-entry-buffer-name)

  ;;update the entry number for this buffer
  (setq thesaurus-current-entry number)

  (thesaurus-entry-mode)

  (display-buffer thesaurus-entry-buffer-name))

(defun thesaurus-wash-data()
  "Remove all text that would end up causing false hits."

  (set-buffer thesaurus-data-buffer-name)


  (save-excursion
    (goto-char (point-min))
  
    ;;remove the header info

    (save-excursion

      (if (re-search-forward "^==========================================================================$"
                             nil t)
          (progn 
            (assert (re-search-forward "^     #1\\." nil t)
                    nil "Could not find first record")
          
            (delete-region (point-min) (match-beginning 0)))))

    ;;wash out all comments...
    ;; <--  p.  2  -->

    (save-excursion
      (while (search-forward "<--" nil t)
        (let(start end)

          (setq start (match-beginning 0))

          (search-forward "-->" nil t)

          (setq end (match-end 0))

          (delete-region start end))))

    ;; %
    ;;      3.  Indeterminate Number
    ;; %

    (save-excursion
      (while (re-search-forward "^%$" nil t)
        (let(start end)

          (setq start (match-beginning 0))

          (re-search-forward "^%$" nil t)

          (setq end (match-end 0))

          (delete-region start end))))))


(define-derived-mode thesaurus-result-list-mode fundamental-mode "ThesaurusResults"
  "Major mode for viewing thesaurus results."

  (toggle-read-only 1)

  (goto-char (point-min))

  (font-lock-mode 1))

(define-derived-mode thesaurus-entry-mode fundamental-mode "ThesaurusEntry"
  "Major mode for viewing thesaurus entries."

  (goto-char (point-min))

  (toggle-read-only 1)
  
  (font-lock-mode 1))


;;setup key bindings for the major mode.
(define-key thesaurus-result-list-mode-map  [return] 'thesaurus-result-list-open-or-scroll-at-point)

(font-lock-add-keywords 'thesaurus-result-list-mode  '(("^#[0-9]+"
                                                        (0 'font-lock-variable-name-face append))))

(font-lock-add-keywords 'thesaurus-entry-mode  '(("^     #[0-9]+"
                                                  (0 'font-lock-variable-name-face append))))

(provide 'thesaurus)

;;; thesaurus.el ends here
